package ru.myitschool.work.ui.main

import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.os.Bundle
import android.view.View
import android.widget.Toast
import androidx.fragment.app.Fragment
import androidx.fragment.app.setFragmentResultListener
import androidx.lifecycle.lifecycleScope
import androidx.navigation.fragment.findNavController
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import retrofit2.Retrofit
import retrofit2.converter.gson.GsonConverterFactory
import ru.myitschool.work.R
import ru.myitschool.work.SessionManager
import ru.myitschool.work.api.ApiService
import ru.myitschool.work.core.Constants
import ru.myitschool.work.databinding.FragmentMainBinding
import ru.myitschool.work.ui.qr.scan.QrScanDestination
import java.net.HttpURLConnection
import java.net.URL

class MainFragment : Fragment(R.layout.fragment_main) {
    private var _binding: FragmentMainBinding? = null
    private val binding get() = _binding

    private val apiService: ApiService by lazy {
        Retrofit.Builder()
            .baseUrl(Constants.SERVER_ADDRESS)
            .addConverterFactory(GsonConverterFactory.create())
            .build()
            .create(ApiService::class.java)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        _binding = FragmentMainBinding.bind(view) // Подключаем binding

        setupUI()
        fetchUserData()

        // Проверяем, есть ли результат QR
        checkQrResult()
    }

    private fun checkQrResult() {
        // Слушаем результат QR сканирования
        setFragmentResultListener(QrScanDestination.REQUEST_KEY) { _, bundle ->
            val qrData = QrScanDestination.getDataIfExist(bundle)
            if (qrData != null) {
                // Если данные QR есть, переходим на экран с результатом
                findNavController().navigate(R.id.qrResultFragment)
            }
        }
    }

    private fun setupUI() {
        // Проверяем, что binding не null, прежде чем устанавливать слушателей
        binding?.apply {
            refresh.setOnClickListener { fetchUserData() }
            logout.setOnClickListener { logout() }
            scan.setOnClickListener { navigateToQrScan() }
        }
    }

    private fun fetchUserData() {
        lifecycleScope.launch {
            showError(null) // Скрыть ошибку, если она была
            try {
                val response = apiService.getUserInfo(SessionManager.userLogin) // Получаем данные пользователя
                if (response.isSuccessful) {
                    response.body()?.let { data ->
                        // Извлекаем значения из Map
                        val fullName = data["name"] as? String ?: "Неизвестно"
                        val position = data["position"] as? String ?: "Неизвестно"
                        val lastVisit = data["lastVisit"] as? String ?: "Неизвестно"
                        val photoUrl = data["photo"] as? String ?: ""

                        // Обновляем UI
                        updateUI(fullName, position, lastVisit, photoUrl)
                    }
                } else {
                    showError(getString(R.string.error_loading_data)) // Показываем ошибку, если данные не загрузились
                }
            } catch (e: Exception) {
                showError(e.localizedMessage) // Показываем ошибку при исключении
            }
        }
    }

    private fun updateUI(fullName: String, position1: String, lastVisit: String, photoUrl: String) {
        // Проверяем, что binding не null, прежде чем обновлять UI
        binding?.apply {
            fullname.text = fullName
            position.text = position1
            lastEntry.text = lastVisit
            if (photoUrl.isNotEmpty()) {
                // Загружаем изображение
                lifecycleScope.launch {
                    val bitmap = loadImageFromUrl(photoUrl)
                    bitmap?.let { photo.setImageBitmap(it) }
                }
            }

            // Показываем элементы
            fullname.visibility = View.VISIBLE
            position.visibility = View.VISIBLE
            lastEntry.visibility = View.VISIBLE
            photo.visibility = View.VISIBLE
            logout.visibility = View.VISIBLE
            scan.visibility = View.VISIBLE
        }
    }

    private suspend fun loadImageFromUrl(urlString: String): Bitmap? {
        return withContext(Dispatchers.IO) {
            try {
                val url = URL(urlString)
                val connection = url.openConnection() as HttpURLConnection
                connection.doInput = true
                connection.connect()
                val inputStream = connection.inputStream
                BitmapFactory.decodeStream(inputStream)
            } catch (e: Exception) {
                e.printStackTrace()
                null
            }
        }
    }

    private fun showError(message: String?) {
        // Проверяем, что binding не null, прежде чем обновлять ошибку
        binding?.apply {
            if (message != null) {
                error.text = message
                error.visibility = View.VISIBLE

                // Скрываем остальные элементы, когда возникает ошибка
                fullname.visibility = View.GONE
                position.visibility = View.GONE
                lastEntry.visibility = View.GONE
                photo.visibility = View.GONE
                logout.visibility = View.GONE
                scan.visibility = View.GONE
            } else {
                error.visibility = View.GONE
            }
        }
    }

    private fun logout() {
        // Очистите данные пользователя
        Toast.makeText(requireContext(), getString(R.string.logged_out), Toast.LENGTH_SHORT).show()
        findNavController().navigate(R.id.loginFragment) // Переход на экран входа
    }

    private fun navigateToQrScan() {
        findNavController().navigate(R.id.qrScanFragment) // Переход на экран сканирования QR
    }

    override fun onDestroyView() {
        _binding = null // Освобождаем binding, когда представление уничтожается
        super.onDestroyView()
    }
}